/**************************************************************************
*** Name of the program:	I2C-Display
*** Function:     			I2C-Display-Routines and Definitions
*** Author:                Andreas Hesse
*** Date of construction:  2004-02-14
*** Date of last revision: 2004-02-19
*** Version:               1.0
*** Comment:
*** The DSM-0822a has 8 character (16 segments per char) and 16 special 
*** signs (for audio application) Example: FM ...
*** You always write the whole display data!
******************************************************************************
***
*** + = Adding new feature
*** - = Bugfix
*** * = Changed
*** ? = detected problem
***
******************************************************************************
***
*** Version management:
*** V 1.0  First construction based on several assembler sources 
*** */

#include <avr/io.h>
#include <avr/pgmspace.h>
#include "i2cmaster.h"
#include "i2cdisp.h"

/******************************************************************************
***		Type for the segmentdefinition (one character on display is 2 bytes value
***		Definitions of segments (16 Segment display)
*** */ 
 
 unsigned char __attribute__ ((progmem)) CharDef[128] =
 {
	0x00, 0x00 , //
	0x01, 0x20,  // ! XXX
	0x80, 0x20 , // "
 	0x25, 0x3d , // # XXX
	0xad, 0xb5 , // $ X
	0xec, 0x37 , // % X
	0x3b, 0x71 , // & XXX
	0x00, 0x02 , // '
	0x00, 0x42 , // (
	0x42, 0x00 , // )
	0x66, 0x66 , // *
	0x24, 0x24 , // +
	0x40, 0x00 , // ,
	0x20, 0x04 , // -
	0x01, 0x00 , // .
	0x40, 0x02 , // /
	0x99, 0x99 , // 0 gleic dem O
	0x00, 0x09 , // 1
	0x39, 0x9C , // 2
	0x29, 0x9D , // 3
   0xA0, 0x0D , // 4
	0xA9, 0x95 , // 5
	0xB9, 0x95 , // 6
	0x08, 0x89 , // 7
	0xB9, 0x9D , // 8
	0xA9, 0x9D , // 9
	0x28, 0x00 , // : XXX
	0x48, 0x00 , // , // XXX
	0x41, 0x12 , // < XX
	0x21, 0x14 , // = XXX
	0x03, 0x50 , // > XX
	0x8c, 0x8c , // ?
	0x3d, 0x99 , // @
	0xb8, 0x8d , // A
	0x0d, 0xbd , // B
	0x99, 0x90 , // C
	0x0d, 0xb9 , // D
	0xb9, 0x94 , // E
	0xb8, 0x80 , // F
	0x99, 0x95 , // G
	0xb0, 0x0d , // H
	0x0d, 0xb0 , // I
	0x1d, 0xa0 , // J
	0xb0, 0x42 , // K
	0x91, 0x10 , // L
	0x92, 0x0b , // M
	0x92, 0x49 , // N
	0x99, 0x99 , // O
	0xb8, 0x8c , // P
	0x99, 0xd9 , // Q
	0xb8, 0xcc , // R
	0xa9, 0x95 , // S
	0x0c, 0xa0 , // T
	0x91, 0x19 , // U
	0xd0, 0x02 , // V
	0xd0, 0x49 , // W
	0x42, 0x42 , // X
	0x06, 0x02 , // Y
	0x49, 0x92 , // Z
	0x99, 0x00 , // [
	0x02, 0x40 , // \ Backslash
	0x00, 0x99 , // ]
	0x40, 0x40 , // ^ XXX
	0x01, 0x10   // _
};

/******************************************************************************
***		Load 16 Segment definition of a ASCII - Character from memory
***		
*** */ 

TCharDef	GetCharDef(int ChrNo)
{
	// Declaration
	TCharDef chrdef;
	
	// Code
	
	// First get position in Table; Note Table starts at SPACE-Character (ASCII 32) 
	ChrNo=ChrNo-32; 		
	
	if ((ChrNo>1) && (ChrNo < 97))
	{
		ChrNo = ChrNo * 2;
	}
	else
	{
		ChrNo = 0;
	}
	chrdef.byte1 = pgm_read_byte_near(&CharDef[ChrNo]);
	ChrNo=ChrNo+1;
	chrdef.byte2 = pgm_read_byte_near(&CharDef[ChrNo]);
	return (chrdef);
} 	// GetCharDef


/******************************************************************************
***		Write characters to display
***		
*** */ 

void Write_To_I2CDisplay (char Message[_No_of_characters])
{
	// Declaration
	int j;
	TCharDef SEGDEF;
	
	// Code
	j=0;
	
	// Check the message byte per byte, get the char definition and write to display
	while ((Message[j] != 0 ) && (j<_No_of_characters))
	{
		SEGDEF = GetCharDef(Message[j]);
		i2c_write(SEGDEF.byte1);
		i2c_write(SEGDEF.byte2); 
		j=j+1;
	};
	
	// write additional "0", "0" to display to clean the rest of the display
	// if less than 8 Char were written
	
	while (j<_No_of_characters)
	{
		i2c_write(0);
		i2c_write(0);
		j=j+1;
	}		
}		// Write_To_I2CDisplay

/******************************************************************************
***		Initialization of display and invoke writing
***		
*** */ 

int I2CDisplay (unsigned char blink, unsigned char specsign1, unsigned char specsign2, char Message[_No_of_characters]) 
{
	// Declaration
	int	ret;
	
	// Code
	
	ret = i2c_start(DSM0822a);       // set device address and write mode
   if ( ret ) 
	{
      // failed to issue start condition, possibly no device found 
      i2c_stop();
      PORTB = 0x02;  // indicate failure
	}
	else 
	{
      // issuing start condition ok, device accessible 
		i2c_write(0xE0); 		// write 224 to select device    
		i2c_write(0xC8); 		// write 200 to enable device   
		i2c_write(blink); 		// write 240 (F0) to set blink mode 
		i2c_write(0x00); 		// write 0 to start with chardefinitions
		Write_To_I2CDisplay (Message);
		i2c_write(specsign1); 		// special signs on display see comment above
		i2c_write(specsign2); 
      i2c_stop();               // set stop conditon = release bus
   };
	return (ret);
}		// I2CDisplay

